@@ -24,6 +24,9 @@ void parse_command_line(int argc, char **argv,
                         struct pkt_info *,
                         struct pkt_info_tcp  *,
                         struct pkt_info_icmp *,
+#if defined(RAIN_IPV6)
+                        struct pkt_info_icmp6 *,
+#endif
                         struct pkt_info_igmp *,
                         u_short *);
 
