@@ -29,6 +29,7 @@ void parse_command_line(int argc, char **argv,
                         u_short *have_target_host)
 {
   int i = 0;
+  int userid = getuid();
   
   while(i < argc) {
     if( (argv[i][0] == '-')  && (strlen(argv[i]) == 2) ) {
@@ -55,9 +56,13 @@ void parse_command_line(int argc, char **argv,
 
         case 'S':
         case 's':{
+          UID_CHECK(userid,argv[i]);
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
+          if((pkt->type == TCPSTREAM) || (pkt->type == TCPCONNECT))
+            DieWithError("streamed connections cannot be spoofed");
           strcpy(pkt->saddr, argv[i + 1]);
           pkt->randip = 0;
+          if(atoi(argv[i + 1]) == 0) pkt->randip = 1;
           break;
         }
         case 'T':
@@ -93,6 +98,7 @@ void parse_command_line(int argc, char **argv,
           pkt->sport = (u_int16_t)atoi(argv[i + 1]);
         }
         else if(strcmp(argv[i], "--sim") == 0) {
+          UID_CHECK(userid,argv[i]);
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
           pkt->simulate = 1;
           pkt->simtype  = atoi(argv[i + 1]);
@@ -124,15 +130,38 @@ void parse_command_line(int argc, char **argv,
           pkt->type = UDP;
         }
         else if(strcmp(argv[i], "--tcp") == 0) {
-          if(pkt->type != DEFAULT) DieWithError("conflicting protocol types");
-          pkt->type = TCP;
+          if((pkt->type != DEFAULT) && (pkt->type != TCPSTREAM) && (pkt->type != TCPCONNECT)) 
+            DieWithError("conflicting protocol types");
+          if((pkt->type == TCPSTREAM) || (pkt->type == TCPCONNECT));
+          else pkt->type = TCP;
         }
-        else if(strcmp(argv[i], "--win") == 0) {
+        else if(strcmp(argv[i], "--stream") == 0) {
+          if((pkt->type != DEFAULT) && (pkt->type != TCP) && (pkt->type != TCPSTREAM) && (pkt->type != TCPCONNECT)) 
+            DieWithError("conflicting protocol types");
+          pkt->type = TCPSTREAM;
+        }
+        else if(strcmp(argv[i], "--connect") == 0) {
+          if((pkt->type != DEFAULT) && (pkt->type != TCP) && (pkt->type != TCPSTREAM) && (pkt->type != TCPCONNECT)) 
+            DieWithError("conflicting protocol types");
+          pkt->type = TCPCONNECT;
+        }
+        else if(strcmp(argv[i], "--seq") == 0) {
+          UID_CHECK(userid,argv[i]);
+          if((pkt->type != DEFAULT) && (pkt->type != TCP)) DieWithError("conflicting protocol types");
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
           pkt->type = TCP;
-          pkt->window = (u_int16_t)atoi(argv[i + 1]);
+          pkttcp->seq  = (u_int16_t)atoi(argv[i + 1]);
+        }
+        else if(strcmp(argv[i], "--win") == 0) {
+          UID_CHECK(userid,argv[i]);
+          if((pkt->type != DEFAULT) && (pkt->type != TCP)) DieWithError("conflicting protocol types");
+          if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
+          pkt->type = TCP;
+          pkttcp->window = (u_int16_t)atoi(argv[i + 1]);
         }
         else if(strcmp(argv[i], "--urg") == 0) {
+          UID_CHECK(userid,argv[i]);
+          if((pkt->type != DEFAULT) && (pkt->type != TCP)) DieWithError("conflicting protocol types");
           if(pkttcp->beenset == 1) DieWithError("conflicting TCP flags");
           pkt->type = TCP;
           pkttcp->syn = 0; /* turn off default */
@@ -140,6 +169,8 @@ void parse_command_line(int argc, char **argv,
           pkttcp->urg = 1;
         }
         else if(strcmp(argv[i], "--ack") == 0) {
+          UID_CHECK(userid,argv[i]);
+          if((pkt->type != DEFAULT) && (pkt->type != TCP)) DieWithError("conflicting protocol types");
           if(pkttcp->beenset == 1) DieWithError("conflicting TCP flags");
           if(pkt->simulate == 1) DieWithError("arguments conflict with simulation mode");
           pkt->type = TCP;
@@ -147,7 +178,16 @@ void parse_command_line(int argc, char **argv,
           pkttcp->beenset = 1;
           pkttcp->ack = 1;
         }
+        else if(strcmp(argv[i], "--ack-seq") == 0) {
+          UID_CHECK(userid,argv[i]);
+          if((pkt->type != DEFAULT) && (pkt->type != TCP)) DieWithError("conflicting protocol types");
+          if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
+          pkt->type = TCP;
+          pkttcp->ack_seq  = atoi(argv[i + 1]);
+        }
         else if(strcmp(argv[i], "--psh") == 0) {
+          UID_CHECK(userid,argv[i]);
+          if((pkt->type != DEFAULT) && (pkt->type != TCP)) DieWithError("conflicting protocol types");
           if(pkttcp->beenset == 1) DieWithError("conflicting TCP flags");
           pkt->type = TCP;
           pkttcp->syn = 0; /* turn off default */
@@ -155,6 +195,8 @@ void parse_command_line(int argc, char **argv,
           pkttcp->psh = 1;
         }
         else if(strcmp(argv[i], "--rst") == 0) {
+          UID_CHECK(userid,argv[i]);
+          if((pkt->type != DEFAULT) && (pkt->type != TCP)) DieWithError("conflicting protocol types");
           if(pkttcp->beenset == 1) DieWithError("conflicting TCP flags");
           pkt->type = TCP;
           pkttcp->syn = 0; /* turn off default */
@@ -162,6 +204,8 @@ void parse_command_line(int argc, char **argv,
           pkttcp->rst = 1;
         }
         else if(strcmp(argv[i], "--syn") == 0) {
+          UID_CHECK(userid,argv[i]);
+          if((pkt->type != DEFAULT) && (pkt->type != TCP)) DieWithError("conflicting protocol types");
           if(pkttcp->beenset == 1) DieWithError("conflicting TCP flags");
           pkt->type = TCP;
           pkttcp->syn = 0; /* turn off default */
@@ -169,6 +213,8 @@ void parse_command_line(int argc, char **argv,
           pkttcp->syn = 1;
         }
         else if(strcmp(argv[i], "--fin") == 0) {
+          UID_CHECK(userid,argv[i]);
+          if((pkt->type != DEFAULT) && (pkt->type != TCP)) DieWithError("conflicting protocol types");
           if(pkttcp->beenset == 1) DieWithError("conflicting TCP flags");
           pkt->type = TCP;
           pkttcp->syn = 0; /* turn off default */
@@ -176,57 +222,70 @@ void parse_command_line(int argc, char **argv,
           pkttcp->fin = 1;
         }      
         else if(strcmp(argv[i], "--icmp") == 0) {
+          UID_CHECK(userid,argv[i]);
           if(pkt->type != DEFAULT) DieWithError("conflicting protocol types");
           pkt->type = ICMP;
         }
         else if(strcmp(argv[i], "--icmp-type") == 0) {
+          UID_CHECK(userid,argv[i]);
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
           pkticmp->type = atoi(argv[i + 1]);
         }
         else if(strcmp(argv[i], "--icmp-code") == 0) {
+          UID_CHECK(userid,argv[i]);
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
           pkticmp->code = atoi(argv[i + 1]);
         }
 #if defined(RAIN_IPV6)
         else if(strcmp(argv[i], "--icmp6-type") == 0) {
+          UID_CHECK(userid,argv[i]);
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
           pkticmp6->type = atoi(argv[i + 1]);
         }
         else if(strcmp(argv[i], "--icmp6-code") == 0) {
+          UID_CHECK(userid,argv[i]);
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
           pkticmp6->code = atoi(argv[i + 1]);
         }
 #endif
         else if(strcmp(argv[i], "--igmp") == 0) {
+          UID_CHECK(userid,argv[i]);
           if(pkt->type != DEFAULT) DieWithError("conflicting protocol types");
           pkt->type = IGMP;
         }
         else if(strcmp(argv[i], "--igmp-type") == 0) {
+          UID_CHECK(userid,argv[i]);
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
           pktigmp->type = atoi(argv[i + 1]);
         }
         else if(strcmp(argv[i], "--igmp-code") == 0) {
+          UID_CHECK(userid,argv[i]);
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
           pktigmp->code = (u_int8_t)atoi(argv[i + 1]);
         }
         else if(strcmp(argv[i], "--frag") == 0) {
+          UID_CHECK(userid,argv[i]);
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
           pkt->frag = 1;
           pkt->frag_off = (u_int16_t)atoi(argv[i + 1]);
         }
         else if(strcmp(argv[i], "--tos") == 0) {
+          UID_CHECK(userid,argv[i]);
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
           pkt->tos = (u_short)atoi(argv[i + 1]);
         }
         else if(strcmp(argv[i], "--ttl") == 0) {
+          UID_CHECK(userid,argv[i]);
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
           pkt->ttl = (u_int8_t)atoi(argv[i + 1]);
         }
         else if(strcmp(argv[i], "--id") == 0) {
+          UID_CHECK(userid,argv[i]);
           if((argv[i + 1] == NULL) || (argv[i + 1][0] == '-')) DieWithRequire(argv[i]);
           pkt->id = atoi(argv[i + 1]);
         }
         else if(strcmp(argv[i], "--id-of") == 0) {
+          UID_CHECK(userid,argv[i]);
           pkt->id_of = 1;
         }
         else if(strcmp(argv[i], "--version") == 0) {
